// Fill out your copyright notice in the Description page of Project Settings.


#include "LMSpawnPosition.h"
#include "Kismet/GameplayStatics.h"
#include "Engine/World.h"
#include "DrawDebugHelpers.h"
#include "Camera/CameraComponent.h"

// Sets default values
ALMSpawnPosition::ALMSpawnPosition()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

}

bool ALMSpawnPosition::CanSpawn()
{
	return !IsPlayerVisible() && !IsOnCooldow;
}

// Called when the game starts or when spawned
void ALMSpawnPosition::BeginPlay()
{
	Super::BeginPlay();
	AActor* Player = UGameplayStatics::GetPlayerPawn(GetWorld(), 0);
	if (Player)
	{
		PlayerCamera = Player->FindComponentByClass<UCameraComponent>();
	}
}

// Called every frame
void ALMSpawnPosition::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
	//UE_LOG(LogTemp, Warning, TEXT("CanSpawn: %s"), CanSpawn() ? TEXT("true") : TEXT("false"));
}

bool ALMSpawnPosition::IsPlayerVisible() const
{
	AActor* Player = UGameplayStatics::GetPlayerPawn(GetWorld(), 0);
	if (!Player) return false;

	FVector Start = GetActorLocation();
	FVector End = PlayerCamera->GetComponentLocation();

	FHitResult HitResult;
	FCollisionQueryParams Params;
	Params.AddIgnoredActor(this);

	bool bHit = GetWorld()->LineTraceSingleByChannel(
		HitResult,
		Start,
		End,
		ECC_Visibility,
		Params
	);

	// Debug trace
	//DrawDebugLine(GetWorld(), Start, End, bHit ? FColor::Red : FColor::Green, false, 2.0f);

	if (bHit)
	{
		//UE_LOG(LogTemp, Warning, TEXT("LineTrace Hit: %s"), *HitResult.GetActor()->GetName());
		return (HitResult.GetActor() == Player && !IsOnCooldow); // Retourne true uniquement si l'acteur touché est le joueur
	}

	//UE_LOG(LogTemp, Warning, TEXT("LineTrace: No Hit"));
	return false; // Aucun obstacle, le joueur est visible
}

void ALMSpawnPosition::StarCooldown()
{
	IsOnCooldow = true;
	//UE_LOG(LogTemp, Warning, TEXT("Spawner Cooldown"));
	GetWorld()->GetTimerManager().SetTimer(ProcessCooldown, this, &ALMSpawnPosition::EndCooldown, TimerCooldown, false);
}

void ALMSpawnPosition::EndCooldown()
{
	IsOnCooldow = false;
	//UE_LOG(LogTemp, Warning, TEXT("Spawner End Cooldown"));
}


